//
// D3DXFont_Intro.cpp - Introduction to D3DXFont
//
// Copyright 2004 by Ken Paulson
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the Drunken Hyena License.  If a copy of the license was
// not included with this software, you may get a copy from:
// http://www.drunkenhyena.com/docs/DHLicense.txt
//
#define _CRT_SECURE_NO_WARNINGS
#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <D3DX9.h>
#include "../common/dhWindow.h"
#include "../common/dhD3D.h"
#include "../common/dhUtility.h"
#include "../Common/dhUserPrefsDialog.h"
#include "../Common/dhFPSTimer.h"

#pragma comment(lib,"d3d9.lib")
#pragma comment(lib,"dxerr9.lib")
#ifdef _DEBUG
  #pragma comment(lib,"d3dx9d.lib")
#else
  #pragma comment(lib,"d3dx9.lib")
#endif

// Forward declarations for all of our functions, see their definitions for more detail
LRESULT CALLBACK default_window_proc(HWND p_hwnd,UINT p_msg,WPARAM p_wparam,LPARAM p_lparam);
HRESULT init_scene(void);
void kill_scene(void);
HRESULT render(void);

// The name of our application.  Used for window and MessageBox titles and error reporting
const char *g_app_name="Introduction to D3DXFont";

// Our screen/window sizes and bit depth.  A better app would allow the user to choose the
// sizes.  I'll do that in a later tutorial, for now this is good enough.
const int g_width=640;
const int g_height=480;
const int g_depth=16; //16-bit colour

// Our global flag to track whether we should quit or not.  When it becomes true, we clean
// up and exit.
bool g_app_done=false;

// Our main Direct3D interface, it doesn't do much on its own, but all the more commonly
// used interfaces are created by it.  It's the first D3D object you create, and the last
// one you release.
IDirect3D9 *g_D3D=NULL;

// The D3DDevice is your main rendering interface.  It represents the display and all of its
// capabilities.  When you create, modify, or render any type of resource, you will likely
// do it through this interface.
IDirect3DDevice9 *g_d3d_device=NULL;

//Our presentation parameters.  They get set in our call to dhInitDevice, and we need them
//in case we need to reset our application.
D3DPRESENT_PARAMETERS g_pp;

//Our FPS timer
dhFPSTimer g_timer(g_app_name);

//Our font interface
ID3DXFont *g_font=NULL;

//A large chunk of text to display
char g_text[4096];

//******************************************************************************************
// Function:WinMain
// Whazzit:The entry point of our application
//******************************************************************************************
int APIENTRY WinMain(HINSTANCE ,HINSTANCE ,LPSTR ,int ){
bool fullscreen;
HWND window=NULL;
D3DFORMAT format;
HRESULT hr;
dhUserPrefs user_prefs(g_app_name);

   dhLogErase();  //Erase the log file to start fresh
   
   // Prompt the user for their preferences
   if (!user_prefs.QueryUser()) {
      dhLog("Exiting\n");
      return 0;
   }

   fullscreen = user_prefs.GetFullscreen();


   // Build our window.
   hr=dhInitWindow(fullscreen,g_app_name,g_width,g_height,default_window_proc,&window);
   if(FAILED(hr)){
      dhLog("Failed to create Window",hr);
      return 0;
   }

   //Build the D3D object
   hr=dhInitD3D(&g_D3D);
   if(FAILED(hr)){
      dhKillWindow(&window);
      dhLog("Failed to create D3D",hr);
      return 0;
   }

   //Find a good display/pixel format
   hr=dhGetFormat(g_D3D,fullscreen,g_depth,&format);
   if(FAILED(hr)){
      dhKillWindow(&window);
      dhLog("Failed to get a display format",hr);
      return 0;
   }

   DWORD adapter = user_prefs.GetAdapter();
   D3DDEVTYPE dev_type = user_prefs.GetDeviceType();

   //Initialize our PresentParameters
   dhInitPresentParameters(fullscreen,window,g_width,g_height,format,D3DFMT_UNKNOWN,&g_pp);

   //Create our device
   hr=dhInitDevice(g_D3D,adapter,dev_type,window,&g_pp,&g_d3d_device);
   if(FAILED(hr)){
      dhKillD3D(&g_D3D,&g_d3d_device);
      dhKillWindow(&window);
      dhLog("Failed to create the device",hr);
      return 0;
   }

   //One-time preparation of objects and other stuff required for rendering
   hr=init_scene();
   if(FAILED(hr)){
      return 0;
   }

   //Loop until the user aborts (closes the window,presses the left mouse button or hits a key)
   while(!g_app_done){

      dhMessagePump();   //Check for window messages

      hr=g_d3d_device->TestCooperativeLevel();

      if(SUCCEEDED(hr)){
         hr=render();   //Draw our incredibly cool graphics
      }

      //Our device is lost
      if(hr == D3DERR_DEVICELOST || hr == D3DERR_DEVICENOTRESET){

         dhHandleLostDevice(g_d3d_device,&g_pp,hr);

      }else if(FAILED(hr)){ //Any other error
         g_app_done=true;
         dhLog("Error rendering",hr);
      }

   }

   //Free all of our objects and other resources
   kill_scene();

   //Clean up all of our Direct3D objects
   dhKillD3D(&g_D3D,&g_d3d_device);

   //Close down our window
   dhKillWindow(&window);

   //Exit happily
   return 0;
}

//******************************************************************************************
// Function:InitVolatileResources
// Whazzit:Prepare any objects that will not survive a device Reset.  These are initialized
//         separately so they can easily be recreated when we Reset our device.
//******************************************************************************************
void InitVolatileResources(void){

   //Init internal resources
   g_font->OnResetDevice();

}
//******************************************************************************************
// Function:FreeVolatileResources
// Whazzit:Free any of our resources that need to be freed so that we can Reset our device,
//         also used to free these resources at the end of the program run.
//******************************************************************************************
void FreeVolatileResources(void){

   //Free up some internal resources
   g_font->OnLostDevice();

}
//******************************************************************************************
// Function:init_scene
// Whazzit:Prepare any objects required for rendering.
//******************************************************************************************
HRESULT init_scene(void){
HRESULT hr=D3D_OK;
FILE *file;
int count;

   hr=D3DXCreateFont(g_d3d_device,     //D3D Device
                     22,               //Font height
                     0,                //Font width
                     FW_NORMAL,        //Font Weight
                     1,                //MipLevels
                     false,            //Italic
                     DEFAULT_CHARSET,  //CharSet
                     OUT_DEFAULT_PRECIS, //OutputPrecision
                     ANTIALIASED_QUALITY, //Quality
                     DEFAULT_PITCH|FF_DONTCARE,//PitchAndFamily
                     "Arial",          //pFacename,
                     &g_font);         //ppFont

   file=fopen("text.txt","r");
   if(!file){
      dhLog("Error opening file for read");
      return E_FAIL;
   }
   count=fread(g_text,1,4000,file);
   g_text[count]=0;  //Make sure string is NULL terminated
   fclose(file);


   g_timer.StartTimer();


   return hr;

}

//******************************************************************************************
// Function:kill_scene
// Whazzit:Clean up any objects we required for rendering.
//******************************************************************************************
void kill_scene(void){

   g_timer.LogTimer();

   if(g_font){
      g_font->Release();
      g_font=NULL;
   }
}

//******************************************************************************************
// Function:render
// Whazzit:Clears the screen, renders our scene and then presents the results.
//******************************************************************************************
HRESULT render(void){
HRESULT hr;
RECT font_rect={0,0,g_width,g_height};
int font_height;
const char *fps_string;

   //Clear the buffer to our new colour.
   hr=g_d3d_device->Clear(0,  //Number of rectangles to clear, we're clearing everything so set it to 0
                          NULL, //Pointer to the rectangles to clear, NULL to clear whole display
                          D3DCLEAR_TARGET,   //What to clear.  We don't have a Z Buffer or Stencil Buffer
                          0x00000000, //Colour to clear to (AARRGGBB)
                          1.0f,  //Value to clear ZBuffer to, doesn't matter since we don't have one
                          0 );   //Stencil clear value, again, we don't have one, this value doesn't matter
   if(FAILED(hr)){
      return hr;
   }

   //Notify the device that we're ready to render
   hr=g_d3d_device->BeginScene();
   if(FAILED(hr)){
      return hr;
   }

   //Show our frames per second
   fps_string=g_timer.GetFPSString();

   SetRect(&font_rect,0,0,g_width,g_height);

   font_height=g_font->DrawText(NULL,        //pSprite
                                fps_string,  //pString
                                -1,          //Count
                                &font_rect,  //pRect
                                DT_LEFT|DT_NOCLIP,//Format,
                                0xFFFFFFFF); //Color
   if(!font_height){
      dhLog("Error Drawing font");
   }


   //Output a big whack of text
   SetRect(&font_rect,0,30,g_width,g_height);

   font_height=g_font->DrawText(NULL,        //pSprite
                                g_text,      //pString
                                -1,          //Count
                                &font_rect,  //pRect
                                DT_LEFT|DT_WORDBREAK, //Format,
                                0xFFFFFFFF); //Color
   if(!font_height){
      dhLog("Error Drawing font");
   }

   
   //Notify the device that we're finished rendering for this frame
   g_d3d_device->EndScene();

   //Show the results
   hr=g_d3d_device->Present(NULL,  //Source rectangle to display, NULL for all of it
                            NULL,  //Destination rectangle, NULL to fill whole display
                            NULL,  //Target window, if NULL uses device window set in CreateDevice
                            NULL );//Unused parameter, set it to NULL

   g_timer.Present();

   return hr;
}


//******************************************************************************************
// Function:default_window_proc
// Whazzit:This handles any incoming Windows messages and sends any that aren't handled to
//         DefWindowProc for Windows to handle.
//******************************************************************************************
LRESULT CALLBACK default_window_proc(HWND p_hwnd,UINT p_msg,WPARAM p_wparam,LPARAM p_lparam){
   
   switch(p_msg){
      case WM_KEYDOWN:
         if(p_wparam == VK_ESCAPE){ //User hit Escape, end the app
            g_app_done=true;
         }
         return 0;
      case WM_CLOSE:    //User hit the Close Window button, end the app
      case WM_LBUTTONDOWN: //user hit the left mouse button
         g_app_done=true;
         return 0;
   }
   
   return (DefWindowProc(p_hwnd,p_msg,p_wparam,p_lparam));
   
}